import { RemovalPolicy } from 'aws-cdk-lib';
import { IRole } from 'aws-cdk-lib/aws-iam';
import { IFunction } from 'aws-cdk-lib/aws-lambda';
import { ILogGroup } from 'aws-cdk-lib/aws-logs';
import { Construct } from 'constructs';
import { DataZoneCustomAssetTypeProps } from './datazone-custom-asset-type-props';
import { TrackedConstruct } from '../../../utils';
/**
 * Properties for the DataZoneCustomAssetTypeFactory construct
 */
export interface DataZoneCustomAssetTypeFactoryProps {
    /**
     * The DataZone domain identifier
     */
    readonly domainId: string;
    /**
     * The Lambda role used by the custom resource provider
     * @default - A new role is created
     */
    readonly lambdaRole?: IRole;
    /**
     * The removal policy for the custom resource
     * @default RemovalPolicy.RETAIN
     */
    readonly removalPolicy?: RemovalPolicy;
}
/**
 * Interface representing a DataZone custom asset type
 */
export interface CustomAssetType {
    /**
     * The domain identifier of the custom asset type
     */
    readonly domainId: string;
    /**
     * The name of the custom asset type
     */
    readonly name: string;
    /**
     * The project identifier owner of the custom asset type
     */
    readonly projectIdentifier: string;
    /**
     * The revision of the custom asset type
     */
    readonly revision: string;
}
/**
 * Factory construct providing resources to create a DataZone custom asset type.
 *
 * @example
 * new dsf.governance.DataZoneCustomAssetTypeFactory(this, 'CustomAssetTypeFactory', {
 *  domainId: 'aba_dc999t9ime9sss',
 * });
 *
 */
export declare class DataZoneCustomAssetTypeFactory extends TrackedConstruct {
    /**
     * The CloudWatch Logs Log Group for the DataZone custom asset type creation
     */
    readonly createLogGroup: ILogGroup;
    /**
     * The Lambda Function for the DataZone custom asset type creation
     */
    readonly createFunction: IFunction;
    /**
     * The IAM Role for the DataZone custom asset type creation
     */
    readonly createRole: IRole;
    /**
     * The service token for the custom resource
     */
    readonly serviceToken: string;
    private readonly domainId;
    private readonly removalPolicy;
    /**
     * Constructs a new instance of DataZoneCustomAssetTypeFactory
     * @param scope the Scope of the CDK Construct
     * @param id the ID of the CDK Construct
     * @param props The DataZoneCustomAssetTypeFactory properties
     */
    constructor(scope: Construct, id: string, props: DataZoneCustomAssetTypeFactoryProps);
    /**
     * Creates a DataZone custom asset type based on the provided properties
     * @param id the ID of the CDK Construct
     * @param customAssetType the properties of the custom asset type
     * @returns the custom asset type
     */
    createCustomAssetType(id: string, customAssetType: DataZoneCustomAssetTypeProps): CustomAssetType;
}
