import { CfnProjectMembership } from 'aws-cdk-lib/aws-datazone';
import { IRule } from 'aws-cdk-lib/aws-events';
import { IRole } from 'aws-cdk-lib/aws-iam';
import { Key } from 'aws-cdk-lib/aws-kms';
import { IFunction } from 'aws-cdk-lib/aws-lambda';
import { ILogGroup } from 'aws-cdk-lib/aws-logs';
import { Construct } from 'constructs';
import { DataZoneGsrMskDataSourceProps } from './datazone-gsr-msk-datasource-props';
import { TrackedConstruct } from '../../../utils';
/**
 * A DataZone custom data source for MSK (Managed Streaming for Kafka) with integration for Glue Schema Registry.
 * The construct creates assets with the MskTopicAssetType in DataZone based on schema definitions in a Glue Schema Registry.
 * It can be either scheduled or react on Schema Registry events (Create Schema, Update Schema, Create Schema Revision).
 *
 * @example
 * import { Schedule } from 'aws-cdk-lib/aws-events';
 *
 * new dsf.governance.DataZoneGsrMskDataSource(this, 'MskDatasource', {
 *   domainId: 'aba_dc999t9ime9sss',
 *   projectId: '999999b3m5cpz',
 *   registryName: 'MyRegistry',
 *   clusterName: 'MyCluster',
 *   runSchedule: Schedule.cron({ minute: '0', hour: '12' }), // Trigger daily at noon
 *   enableSchemaRegistryEvent: true, // Enable events for Glue Schema Registry changes
 * });
 */
export declare class DataZoneGsrMskDataSource extends TrackedConstruct {
    /**
     * The IAM Role of the Lambda Function interacting with DataZone API to create inventory assets
     */
    readonly lambdaRole: IRole;
    /**
     * The membership of the Lambda Role on the DataZone Project
     */
    readonly dataZoneMembership: CfnProjectMembership;
    /**
     * The Event Bridge Rule for schema creation and update
     */
    readonly createUpdateEventRule?: IRule;
    /**
     * The Event Bridge Rule for trigger the data source execution
     */
    readonly scheduleRule?: IRule;
    /**
     * The Event Bridge Rule for schema deletion
     */
    readonly deleteEventRule?: IRule;
    /**
     * The Log Group for the Lambda Function creating DataZone Inventory Assets
     */
    readonly lambdaLogGroup: ILogGroup;
    /**
     * The Lambda Function creating DataZone Inventory Assets
     */
    readonly lambdaFunction: IFunction;
    /**
     * The KMS Key used to encrypt the SSM Parameter storing assets information
     */
    readonly ssmParameterKey: Key;
    private readonly removalPolicy;
    /**
     * Build an instance of the DataZoneGsrMskDataSource
     * @param scope the Scope of the CDK Construct
     * @param id the ID of the CDK Construct
     * @param props The DataZoneGsrMskDataSourceProps properties
     */
    constructor(scope: Construct, id: string, props: DataZoneGsrMskDataSourceProps);
}
