import { IRule } from 'aws-cdk-lib/aws-events';
import { IRole, Role } from 'aws-cdk-lib/aws-iam';
import { IKey } from 'aws-cdk-lib/aws-kms';
import { IFunction } from 'aws-cdk-lib/aws-lambda';
import { ILogGroup } from 'aws-cdk-lib/aws-logs';
import { IQueue } from 'aws-cdk-lib/aws-sqs';
import { StateMachine } from 'aws-cdk-lib/aws-stepfunctions';
import { Construct } from 'constructs';
import { DataZoneMskCentralAuthorizerProps } from './datazone-msk-central-authorizer-props';
import { TrackedConstruct } from '../../../utils';
/**
 * A central authorizer workflow for granting read access to Kafka topics.
 * The workflow is triggered by an event sent to the DataZone event bus.
 * First, it collects metadata from DataZone about the Kafka topics.
 * Then, it grants access to the relevant IAM roles.
 * Finally acknowledge the subscription grant in DataZone.
 *
 * @example
 * new dsf.governance.DataZoneMskCentralAuthorizer(this, 'MskAuthorizer', {
 *   domainId: 'aba_dc999t9ime9sss',
 * });
 */
export declare class DataZoneMskCentralAuthorizer extends TrackedConstruct {
    /**
     * The name of the authorizer
     */
    static readonly AUTHORIZER_NAME = "MskTopicAuthorizer";
    /**
     * The asset type for the DataZone custom asset type
     */
    static readonly MSK_ASSET_TYPE = "MskTopicAssetType";
    private static DEFAULT_LOGS_RETENTION;
    /**
     * The role used to collect metadata from DataZone
     */
    readonly metadataCollectorRole: IRole;
    /**
     * The Lambda function used to collect metadata from DataZone
     */
    readonly metadataCollectorFunction: IFunction;
    /**
     * The Cloudwatch Log Group for logging the metadata collector
     */
    readonly metadataCollectorLogGroup: ILogGroup;
    /**
     * The role used to acknowledge the subscription grant in DataZone
     */
    readonly datazoneCallbackRole: IRole;
    /**
     * The Lambda function used to acknowledge the subscription grant in DataZone
     */
    readonly datazoneCallbackFunction: IFunction;
    /**
     * The Cloudwatch Log Group for logging the datazone callback
     */
    readonly datazoneCallbackLogGroup: ILogGroup;
    /**
     * The role used by the DataZone event to trigger the authorizer workflow
     */
    readonly datazoneEventRole: IRole;
    /**
     * The event rule used to trigger the authorizer workflow
     */
    readonly datazoneEventRule: IRule;
    /**
     * The state machine used to orchestrate the authorizer workflow
     */
    readonly stateMachine: StateMachine;
    /**
     * The IAM Role used by the authorizer workflow State Machine
     */
    readonly stateMachineRole: Role;
    /**
     * The IAM Role used by the authorizer workflow callback
     */
    readonly stateMachineCallbackRole: Role;
    /**
     * The CloudWatch Log Group used to log the authorizer state machine
     */
    stateMachineLogGroup: ILogGroup;
    /**
     * The SQS Queue used as a dead letter queue for the authorizer workflow
     */
    readonly deadLetterQueue: IQueue;
    /**
     * The key used to encrypt the dead letter queue
     */
    readonly deadLetterKey: IKey;
    private readonly removalPolicy;
    /**
     * Construct an instance of the DataZoneMskCentralAuthorizer
     * @param scope the Scope of the CDK Construct
     * @param id the ID of the CDK Construct
     * @param props The DataZoneMskCentralAuthorizer properties
     */
    constructor(scope: Construct, id: string, props: DataZoneMskCentralAuthorizerProps);
    /**
     * Connect the central authorizer workflow with environment authorizer workflows in other accounts.
     * This method grants the environment workflow to send events in the default Event Bridge bus for orchestration.
     * @param id The construct ID to use
     * @param accountId The account ID to register the authorizer with
     * @returns The CfnEventBusPolicy created to grant the account
     */
    registerAccount(id: string, accountId: string): void;
}
