import { IManagedPolicy, IRole } from 'aws-cdk-lib/aws-iam';
import { IBucket } from 'aws-cdk-lib/aws-s3';
import { Construct } from 'constructs';
import { PySparkApplicationPackageProps } from './pyspark-application-package-props';
import { AccessLogsBucket } from '../../../storage';
import { TrackedConstruct } from '../../../utils';
/**
 * A construct that takes your PySpark application, packages its virtual environment and uploads it along its entrypoint to an Amazon S3 bucket
 * This construct requires Docker daemon installed locally to run.
 * @see https://awslabs.github.io/data-solutions-framework-on-aws/docs/constructs/library/Processing/pyspark-application-package
 *
 * @example
 * let pysparkPacker = new dsf.processing.PySparkApplicationPackage (this, 'pysparkPacker', {
 *   applicationName: 'my-pyspark',
 *   entrypointPath: '/Users/my-user/my-spark-job/app/app-pyspark.py',
 *   dependenciesFolder: '/Users/my-user/my-spark-job/app',
 *   removalPolicy: cdk.RemovalPolicy.DESTROY,
 * });
 */
export declare class PySparkApplicationPackage extends TrackedConstruct {
    /**
     * The prefix used to store artifacts on the artifact bucket
     */
    static readonly ARTIFACTS_PREFIX = "emr-artifacts";
    /**
     * The location (generally it's an S3 URI) where the entry point is saved.
     * You can pass this location to your Spark job.
     */
    readonly entrypointUri: string;
    /**
     * The location (generally an S3 URI) where the archive of the Python virtual environment with all dependencies is stored.
     * You can pass this location to your Spark job.
     */
    readonly venvArchiveUri?: string;
    /**
     * The Spark Config containing the configuration of virtual environment archive with all dependencies.
     */
    readonly sparkVenvConf?: string;
    /**
     * The S3 Bucket for storing the artifacts (entrypoint and virtual environment archive).
     */
    readonly artifactsBucket: IBucket;
    /**
     * The IAM Role used by the BucketDeployment to upload the artifacts to an s3 bucket.
     * In case you provide your own S3 Bucket for storing the artifacts (entrypoint and virtual environment archive),
     * you must provide S3 write access to this role to upload the artifacts.
     */
    readonly assetUploadRole: IRole;
    /**
     * The IAM Managed Policy used by the custom resource for the assets deployment
     */
    readonly assetUploadManagedPolicy: IManagedPolicy;
    /**
     * The access logs bucket to log accesses on the artifacts bucket
     */
    readonly artifactsAccessLogsBucket?: AccessLogsBucket;
    /**
     * @param {Construct} scope the Scope of the CDK Construct
     * @param {string} id the ID of the CDK Construct
     * @param props {@link PySparkApplicationPackageProps}
     */
    constructor(scope: Construct, id: string, props: PySparkApplicationPackageProps);
}
