import { CustomResource } from 'aws-cdk-lib';
import { IGatewayVpcEndpoint, ISecurityGroup, IVpc } from 'aws-cdk-lib/aws-ec2';
import { Cluster, KubernetesVersion } from 'aws-cdk-lib/aws-eks';
import { CfnVirtualCluster } from 'aws-cdk-lib/aws-emrcontainers';
import { IRule } from 'aws-cdk-lib/aws-events';
import { CfnServiceLinkedRole, IManagedPolicy, IRole, Role } from 'aws-cdk-lib/aws-iam';
import { IKey } from 'aws-cdk-lib/aws-kms';
import { ILogGroup } from 'aws-cdk-lib/aws-logs';
import { IBucket } from 'aws-cdk-lib/aws-s3';
import { IQueue } from 'aws-cdk-lib/aws-sqs';
import { Construct } from 'constructs';
import { EmrVirtualClusterProps } from './emr-virtual-cluster-props';
import { SparkEmrContainersRuntimeInteractiveSessionProps } from './spark-emr-containers-interactive-endpoint-props';
import { SparkEmrContainersRuntimeProps } from './spark-emr-containers-runtime-props';
import { TrackedConstruct } from '../../../../utils';
/**
 * A construct to create an EKS cluster, configure it and enable it with EMR on EKS
 * @see https://awslabs.github.io/data-solutions-framework-on-aws/docs/constructs/library/Processing/spark-emr-containers-runtime
 *
 * @example
 * import { ManagedPolicy, PolicyDocument, PolicyStatement } from 'aws-cdk-lib/aws-iam';
 * import { KubectlV30Layer } from '@aws-cdk/lambda-layer-kubectl-v30';
 *
 * const kubectlLayer = new KubectlV30Layer(this, 'kubectlLayer');
 *
 * const emrEksCluster = dsf.processing.SparkEmrContainersRuntime.getOrCreate(this, {
 *   publicAccessCIDRs: ['10.0.0.0/16'],
 *   kubectlLambdaLayer: kubectlLayer,
 * });
 *
 * const virtualCluster = emrEksCluster.addEmrVirtualCluster(this, {
 *   name: 'example',
 *   createNamespace: true,
 *   eksNamespace: 'example',
 * });
 *
 * const s3Read = new PolicyDocument({
 *   statements: [new PolicyStatement({
 *     actions: [
 *       's3:GetObject',
 *     ],
 *     resources: ['arn:aws:s3:::aws-data-analytics-workshop'],
 *   })],
 * });
 *
 * const s3ReadPolicy = new ManagedPolicy(this, 's3ReadPolicy', {
 *   document: s3Read,
 * });
 *
 * const execRole = emrEksCluster.createExecutionRole(this, 'ExecRole', s3ReadPolicy, 'example', 's3ReadExecRole');
 */
export declare class SparkEmrContainersRuntime extends TrackedConstruct {
    /**
     *  The default EMR on EKS version
     */
    static readonly DEFAULT_EMR_EKS_VERSION: import("../../emr-releases").EmrContainersRuntimeVersion;
    /**
     * The default EKS version
     */
    static readonly DEFAULT_EKS_VERSION: KubernetesVersion;
    /**
     * The default name of the EKS cluster
     */
    static readonly DEFAULT_CLUSTER_NAME = "data-platform";
    /**
     * The default CIDR when the VPC is created
     */
    static readonly DEFAULT_VPC_CIDR = "10.0.0.0/16";
    /**
     * A static method granting the right to start and monitor a job to an IAM Role.
     * The method will scope the following actions `DescribeJobRun`, `TagResource` and `ListJobRuns` to the provided virtual cluster.
     * It will also scope `StartJobRun` as defined in the
     * [EMR on EKS official documentation](https://docs.aws.amazon.com/emr/latest/EMR-on-EKS-DevelopmentGuide/iam-execution-role.html)
     *
     * @param startJobRole the role that will call the start job api and which needs to have the iam:PassRole permission
     * @param executionRoleArn the role used by EMR on EKS to access resources during the job execution
     * @param virtualClusterArn the EMR Virtual Cluster ARN to which the job is submitted
     */
    static grantStartJobExecution(startJobRole: IRole, executionRoleArn: string[], virtualClusterArn: string): void;
    /**
     * Get an existing EmrEksCluster based on the cluster name property or create a new one
     * only one EKS cluster can exist per stack
     * @param {Construct} scope the CDK scope used to search or create the cluster
     * @param {EmrEksClusterProps} props the EmrEksClusterProps [properties]{@link EmrEksClusterProps} if created
     */
    static getOrCreate(scope: Construct, props: SparkEmrContainersRuntimeProps): SparkEmrContainersRuntime;
    /**
     * The EKS cluster created by the construct if it is not provided
     */
    readonly eksCluster: Cluster;
    /**
     * The IAM Role used by IRSA for the aws-node daemonset
     */
    readonly awsNodeRole?: IRole;
    /**
     * The IAM role used by the tooling managed nodegroup hosting core Kubernetes controllers
     * like EBS CSI driver, core dns
     */
    readonly ec2InstanceNodeGroupRole: IRole;
    /**
     * The SQS queue used by Karpenter to receive critical events from AWS services which may affect your nodes.
     */
    readonly karpenterQueue?: IQueue;
    /**
     * The security group used by the EC2NodeClass of the default nodes
     */
    readonly karpenterSecurityGroup?: ISecurityGroup;
    /**
     * The rules used by Karpenter to track node health, rules are defined in the cloudformation below
     * https://raw.githubusercontent.com/aws/karpenter/"${KARPENTER_VERSION}"/website/content/en/preview/getting-started/getting-started-with-karpenter/cloudformation.yaml
     */
    readonly karpenterEventRules?: Array<IRule>;
    /**
     * The configuration override for the spark application to use with the default nodes dedicated for notebooks
     */
    readonly notebookDefaultConfig?: string;
    /**
     * The configuration override for the spark application to use with the default nodes for criticale jobs
     */
    readonly criticalDefaultConfig?: string;
    /**
     * The configuration override for the spark application to use with the default nodes for none criticale jobs
     */
    readonly sharedDefaultConfig?: string;
    /**
     * The bucket holding podtemplates referenced in the configuration override for the job
     */
    readonly assetBucket?: IBucket;
    /**
     * The S3 location holding the driver pod tempalte for critical nodes
     */
    readonly podTemplateS3LocationCriticalDriver?: string;
    /**
     * The S3 location holding the executor pod tempalte for critical nodes
     */
    readonly podTemplateS3LocationCriticalExecutor?: string;
    /**
     * The S3 location holding the driver pod tempalte for shared nodes
     */
    readonly podTemplateS3LocationDriverShared?: string;
    /**
     * The S3 location holding the executor pod tempalte for shared nodes
     */
    readonly podTemplateS3LocationExecutorShared?: string;
    /**
     * The S3 location holding the driver pod tempalte for interactive sessions
     */
    readonly podTemplateS3LocationNotebookDriver?: string;
    /**
     * The S3 location holding the executor pod tempalte for interactive sessions
     */
    readonly podTemplateS3LocationNotebookExecutor?: string;
    /**
     * The IAM Role created for the EBS CSI controller
     */
    readonly csiDriverIrsaRole?: IRole;
    /**
     * The IAM role created for the Karpenter controller
     */
    readonly karpenterIrsaRole?: IRole;
    /**
     * The VPC used by the EKS cluster
     */
    readonly vpc: IVpc;
    /**
     * The KMS Key used for the VPC flow logs when the VPC is created
     */
    readonly flowLogKey?: IKey;
    /**
     * The IAM Role used for the VPC flow logs when the VPC is created
     */
    readonly flowLogRole?: IRole;
    /**
     * The S3 VPC endpoint attached to the private subnets of the VPC when VPC is created
     */
    readonly s3VpcEndpoint?: IGatewayVpcEndpoint;
    /**
     * The CloudWatch Log Group for the VPC flow log when the VPC is created
     */
    readonly flowLogGroup?: ILogGroup;
    /**
     * The Service Linked role created for EMR
     */
    readonly emrServiceRole?: CfnServiceLinkedRole;
    /**
     * The KMS key used for storing EKS secrets
     */
    readonly eksSecretKmsKey?: IKey;
    /**
     * The IAM role used to upload assets (pod templates) on S3
     */
    readonly assetUploadBucketRole?: IRole;
    private readonly karpenterChart?;
    private readonly defaultNodes?;
    private readonly createEmrOnEksServiceLinkedRole?;
    private readonly podTemplateLocation;
    private readonly podTemplatePolicy;
    private readonly removalPolicy;
    private readonly interactiveSessionsProviderServiceToken;
    /**
     * Constructs a new instance of the EmrEksCluster construct.
     * @param {Construct} scope the Scope of the CDK Construct
     * @param {string} id the ID of the CDK Construct
     * @param {EmrEksClusterProps} props the EmrEksClusterProps [properties]{@link EmrEksClusterProps}
     */
    private constructor();
    /**
     * Add a new Amazon EMR Virtual Cluster linked to Amazon EKS Cluster.
     * @param {Construct} scope of the stack where virtual cluster is deployed
     * @param {EmrVirtualClusterProps} options the EmrVirtualClusterProps [properties]{@link EmrVirtualClusterProps}
     */
    addEmrVirtualCluster(scope: Construct, options: EmrVirtualClusterProps): CfnVirtualCluster;
    /**
     * Create and configure a new Amazon IAM Role usable as an execution role.
     * This method makes the created role assumed by the Amazon EKS cluster Open ID Connect provider.
     * @param {Construct} scope of the IAM role
     * @param {string} id of the CDK resource to be created, it should be unique across the stack
     * @param {IManagedPolicy} policy the execution policy to attach to the role
     * @param {string} eksNamespace The namespace from which the role is going to be used. MUST be the same as the namespace of the Virtual Cluster from which the job is submitted
     * @param {string} name Name to use for the role, required and is used to scope the iam role
     */
    createExecutionRole(scope: Construct, id: string, policy: IManagedPolicy, eksNamespace: string, name: string): Role;
    /**
     * Upload podTemplates to the Amazon S3 location used by the cluster.
     * @param {string} id the unique ID of the CDK resource
     * @param {string} filePath The local path of the yaml podTemplate files to upload
     */
    uploadPodTemplate(id: string, filePath: string): void;
    /**
     * Apply the provided manifest and add the CDK dependency on EKS cluster
     * @param {string} id the unique ID of the CDK resource
     * @param {any} manifest The manifest to apply.
     * You can use the Utils class that offers method to read yaml file and load it as a manifest
     */
    addKarpenterNodePoolAndNodeClass(id: string, manifest: any): any;
    /**
     * Creates a new Amazon EMR managed endpoint to be used with Amazon EMR Virtual Cluster .
     * CfnOutput can be customized.
     * @param {Construct} scope the scope of the stack where managed endpoint is deployed
     * @param {string} id the CDK id for endpoint
     * @param {SparkEmrContainersRuntimeInteractiveSessionProps} interactiveSessionOptions the EmrManagedEndpointOptions to configure the Amazon EMR managed endpoint
     */
    addInteractiveEndpoint(scope: Construct, id: string, interactiveSessionOptions: SparkEmrContainersRuntimeInteractiveSessionProps): CustomResource;
}
